"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from .httpclient import AsyncHttpClient, HttpClient
from .sdkconfiguration import SDKConfiguration
from .utils.logger import Logger, get_default_logger
from .utils.retries import RetryConfig
import httpx
from leonardo_ai_sdk import utils
from leonardo_ai_sdk._hooks import SDKHooks
from leonardo_ai_sdk.dataset import Dataset
from leonardo_ai_sdk.elements import Elements
from leonardo_ai_sdk.image import Image
from leonardo_ai_sdk.init_images import InitImages
from leonardo_ai_sdk.models import shared
from leonardo_ai_sdk.models_ import Models
from leonardo_ai_sdk.motion import Motion
from leonardo_ai_sdk.pricing_calculator import PricingCalculator
from leonardo_ai_sdk.prompt import Prompt
from leonardo_ai_sdk.realtime_canvas import RealtimeCanvas
from leonardo_ai_sdk.texture import Texture
from leonardo_ai_sdk.threed_model_assets import ThreeDModelAssets
from leonardo_ai_sdk.types import OptionalNullable, UNSET
from leonardo_ai_sdk.user import User
from leonardo_ai_sdk.variation import Variation
from typing import Any, Callable, Dict, Optional, Union


class LeonardoAiSDK(BaseSDK):
    r"""Rest Endpoints: Leonardo.Ai API OpenAPI specification."""

    init_images: InitImages
    dataset: Dataset
    elements: Elements
    image: Image
    realtime_canvas: RealtimeCanvas
    motion: Motion
    texture: Texture
    user: User
    models: Models
    three_d_model_assets: ThreeDModelAssets
    pricing_calculator: PricingCalculator
    prompt: Prompt
    variation: Variation

    def __init__(
        self,
        bearer_auth: Union[str, Callable[[], str]],
        server_idx: Optional[int] = None,
        server_url: Optional[str] = None,
        url_params: Optional[Dict[str, str]] = None,
        client: Optional[HttpClient] = None,
        async_client: Optional[AsyncHttpClient] = None,
        retry_config: OptionalNullable[RetryConfig] = UNSET,
        timeout_ms: Optional[int] = None,
        debug_logger: Optional[Logger] = None,
    ) -> None:
        r"""Instantiates the SDK configuring it with the provided parameters.

        :param bearer_auth: The bearer_auth required for authentication
        :param server_idx: The index of the server to use for all methods
        :param server_url: The server URL to use for all methods
        :param url_params: Parameters to optionally template the server URL with
        :param client: The HTTP client to use for all synchronous methods
        :param async_client: The Async HTTP client to use for all asynchronous methods
        :param retry_config: The retry configuration to use for all supported methods
        :param timeout_ms: Optional request timeout applied to each operation in milliseconds
        """
        if client is None:
            client = httpx.Client()

        assert issubclass(
            type(client), HttpClient
        ), "The provided client must implement the HttpClient protocol."

        if async_client is None:
            async_client = httpx.AsyncClient()

        if debug_logger is None:
            debug_logger = get_default_logger()

        assert issubclass(
            type(async_client), AsyncHttpClient
        ), "The provided async_client must implement the AsyncHttpClient protocol."

        security: Any = None
        if callable(bearer_auth):
            security = lambda: shared.Security(bearer_auth=bearer_auth())  # pylint: disable=unnecessary-lambda-assignment
        else:
            security = shared.Security(bearer_auth=bearer_auth)

        if server_url is not None:
            if url_params is not None:
                server_url = utils.template_url(server_url, url_params)

        BaseSDK.__init__(
            self,
            SDKConfiguration(
                client=client,
                async_client=async_client,
                security=security,
                server_url=server_url,
                server_idx=server_idx,
                retry_config=retry_config,
                timeout_ms=timeout_ms,
                debug_logger=debug_logger,
            ),
        )

        hooks = SDKHooks()

        current_server_url, *_ = self.sdk_configuration.get_server_details()
        server_url, self.sdk_configuration.client = hooks.sdk_init(
            current_server_url, self.sdk_configuration.client
        )
        if current_server_url != server_url:
            self.sdk_configuration.server_url = server_url

        # pylint: disable=protected-access
        self.sdk_configuration.__dict__["_hooks"] = hooks

        self._init_sdks()

    def _init_sdks(self):
        self.init_images = InitImages(self.sdk_configuration)
        self.dataset = Dataset(self.sdk_configuration)
        self.elements = Elements(self.sdk_configuration)
        self.image = Image(self.sdk_configuration)
        self.realtime_canvas = RealtimeCanvas(self.sdk_configuration)
        self.motion = Motion(self.sdk_configuration)
        self.texture = Texture(self.sdk_configuration)
        self.user = User(self.sdk_configuration)
        self.models = Models(self.sdk_configuration)
        self.three_d_model_assets = ThreeDModelAssets(self.sdk_configuration)
        self.pricing_calculator = PricingCalculator(self.sdk_configuration)
        self.prompt = Prompt(self.sdk_configuration)
        self.variation = Variation(self.sdk_configuration)

    def __enter__(self):
        return self

    async def __aenter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        if self.sdk_configuration.client is not None:
            self.sdk_configuration.client.close()

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        if self.sdk_configuration.async_client is not None:
            await self.sdk_configuration.async_client.aclose()
